from base_st_classes import *

class RenameSuite(NFSSuite):
    """Test operation 29: RENAME

    FIXME: Test renaming of a named attribute
    to be a regular file and vice versa.

    Equivalence partitioning:

    Input Condition: saved filehandle
        Valid equivalence classes:
            dir(10)
        Invalid equivalence classes:
            non-dir(11)
            no filehandle(12)
            invalid filehandle(13)
    Input Condition: oldname
        Valid equivalence classes:
            valid name(20)
        Invalid equivalence classes:
            non-existent name(21)
            zerolength(22)
            non-utf8(23)
    Input Condition: current filehandle
        Valid equivalence classes:
            dir(30)
        Invalid equivalence classes:
            non-dir(31)
            no filehandle(32)
            invalid filehandle(33)
    Input Condition: newname
        Valid equivalence classes:
            valid name(40)
        Invalid equivalence classes:
            zerolength(41)
            non-utf8(42)

    Comments: It's not possible to cover eq. class 32, since saving a filehandle
    gives a current filehandle as well. 
    """
    def setUp(self):
        NFSSuite.setUp(self)
        self.oldname = "object1"
        self.obj_name = self.oldname # Easier call of create_object()
        self.newname = "object2"
        self.types =  [NF4DIR,NF4SOCK,NF4FIFO,NF4LNK,NF4BLK,NF4CHR]
        
    def tearDown(self):
        try: self.clean_dir(self.tmp_dir)
        except SkipException, e:
            pass

    def _rename(self, oldname=None, newname=None, olddir=None, newdir=None,
                error=[NFS4_OK]):
        """ Call rename
        """
        # Set defaults
        if oldname==None: oldname=self.oldname
        if newname==None: newname=self.newname
        if not olddir: olddir = self.tmp_dir
        if not newdir: newdir = self.tmp_dir
        
        # Set (sfh) to olddir
        ops = [self.ncl.putrootfh_op()]
        ops.extend(self.ncl.lookup_path(olddir))
        ops.append(self.ncl.savefh_op())

        # Set (cfh) to newdir
        ops.append(self.ncl.putrootfh_op())
        ops.extend(self.ncl.lookup_path(newdir))
                         
        # Call rename
        renameop = self.ncl.rename_op(oldname, newname)
        ops.append(renameop)
        res = self.ncl.do_ops(ops)
        self.assert_status(res,error)

        if res.status == NFS4_OK:
            return res

    #
    # Testcases covering valid equivalence classes.
    #
    def testValid(self):
        """RENAME : normal operation

        Covered valid equivalence classes: 10, 20, 30, 40
        """
        skip_count = 0
        for type in self.types:
            try:
                self.create_obj(type=type)
                self._rename()
            except SkipException, e:
                skip_count += 1
            try:
                self.remove_object()
            except SkipException, e:
                pass
            try:
                self.remove_object(name=self.newname)
            except SkipException, e:
                pass
        if skip_count == len(self.types):
            self.skip("Could not create any objects")

    #
    # Testcases covering invalid equivalence classes.
    #
    def testSfhNotDir(self):
        """RENAME with non-dir (sfh) should return NFS4ERR_NOTDIR

        Covered invalid equivalence classes: 11
        """
        self._rename(olddir=self.regfile, error=[NFS4ERR_NOTDIR])

    def testNoSfh(self):
        """RENAME without (sfh) should return NFS4ERR_NOFILEHANDLE

        Covered invalid equivalence classes: 12
        """
        # Lookup target directory
        ops = [self.ncl.putrootfh_op()]
        ops.extend(self.ncl.lookup_path(self.tmp_dir))
        
        # Rename
        renameop = self.ncl.rename_op(self.oldname, self.newname)
        ops.append(renameop)
        res = self.ncl.do_ops(ops)
        self.assert_status(res, [NFS4ERR_NOFILEHANDLE])

    # FIXME: Cover eq. class 13.
    # (FRED) How? To get the filehandle to be invalid must first
    #        put it into cfh then save it, both operations causing
    #        errors

    def testNonExistent(self):
        """RENAME on non-existing object should return NFS4ERR_NOENT

        Covered invalid equivalence classes: 21
        """
        self._rename(oldname="vapor_object",error=[NFS4ERR_NOENT])

    def testZeroLengthOldname(self):
        """RENAME with zero length oldname should return NFS4ERR_INVAL

        Covered invalid equivalence classes: 22
        """
        self._rename(oldname="", error=[NFS4ERR_INVAL])

    def testNonUTF8Oldname(self):
        """RENAME with non-UTF8 oldname should return NFS4ERR_INVAL

        Covered invalid equivalence classes: 23

        Comments: There is no need to create the object first; the
        UTF8 check should be done before verifying if the object exists. 
        """
        for name in self.get_invalid_utf8strings():
            self._rename(oldname=name, error=[NFS4ERR_INVAL])
        
    def testCfhNotDir(self):
        """RENAME with non-dir (cfh) should return NFS4ERR_NOTDIR

        Covered invalid equivalence classes: 31
        """
        for path in [ self.regfile,
                      self.blockfile,
                      self.charfile,
                      self.linkfile,
                      self.socketfile,
                      self.fifofile ]:
            self._rename(newdir=path, error=[NFS4ERR_NOTDIR])

    # FIXME: Cover eq. class 33.
    # (FRED) Again, how?
    
    def testZeroLengthNewname(self):
        """RENAME with zero length newname should return NFS4ERR_INVAL

        Covered invalid equivalence classes: 41
        """
        try: self.create_obj()
        except SkipException, e:
            self.skip(e)

        self._rename(newname="", error=[NFS4ERR_INVAL])
    
    def testNonUTF8Newname(self):
        """RENAME with non-UTF8 newname should return NFS4ERR_INVAL

        Covered invalid equivalence classes: 42
        """
        try: self.create_obj()
        except SkipException, e:
            self.skip(e)

        for name in self.get_invalid_utf8strings():
            self._rename(newname=name, error=[NFS4ERR_INVAL])

    #
    # Extra tests. 
    #
    def testDotsOldname(self):
        """RENAME with oldname containing . or .. should return NOENT/INVAL

        Extra test
        
        No files named . or .. should exist in /doc directory. RENAME should
        return NFS4ERR_NOENT (if server supports "." and ".." as file names)
        or NFS4ERR_INVAL. 
        """
        self._rename(oldname=".", error=[NFS4ERR_NOENT, NFS4ERR_INVAL])
        self._rename(oldname="..", error=[NFS4ERR_NOENT, NFS4ERR_INVAL])

    def testDotsNewname(self):
        """RENAME with newname . or .. should succeed or return OK/INVAL

        Extra test
        """
        # Create dummy object
        try: self.create_obj()
        except SkipException, e:
            self.skip(e)

        # Try to rename it to "."
        res = self._rename(newname=".", error=[NFS4_OK, NFS4ERR_INVAL])

        # Create dummy object
        if res :
            try: self.create_obj()
            except SkipException, e:
                self.skip(e)

        # Try to rename it to ".."
        self._rename(newname="..", error=[NFS4_OK, NFS4ERR_INVAL])

    def testNamingPolicy(self):
        """RENAME should obey OPEN file name creation policy

        Extra test
        """
        # This test tests the create part of RENAME. 
        self.init_connection()

        try:
            (x, rejected_names_open) = self.try_file_names(creator=self.create_via_open)
            (x, rejected_names_rename) = self.try_file_names(creator=self.create_via_rename)
            self.failIf(rejected_names_open != rejected_names_rename,
                        "RENAME does not obey OPEN naming policy")
        except SkipException, e:
            self.skip(e)

    def testValidNames(self):
        """RENAME should succeed on all legal names

        Extra test

        Comments: This test tries RENAME on all names returned from try_file_names()
        """
        # This test tests the lookup part of RENAME. 
        self.init_connection()

        # Saved files for 
        try:
            (accepted_names, rejected_names) = self.try_file_names(remove_files=0)
        except SkipException, e:
            self.skip(e)

        # Ok, lets try RENAME on all accepted names
        lookup_dir_ops = self.ncl.lookup_path(self.tmp_dir)
        for filename in accepted_names:
            self._rename(oldname=filename)

    def testInvalidNames(self):
        """RENAME should fail with NFS4ERR_NOENT on all unexisting, invalid file names

        Extra test

        (FRED) - see comment on same test in st_lookup
        
        Comments: Tries RENAME on rejected file names from try_file_names().
        NFS4ERR_INVAL should NOT be returned in this case, although
        the server rejects creation of objects with these names
        """
        self.init_connection()

        try:
            (accepted_names, rejected_names) = self.try_file_names()
        except SkipException, e:
            self.skip(e)

        # Ok, lets try RENAME on all rejected names
        lookup_dir_ops = self.ncl.lookup_path(self.tmp_dir)
        for filename in rejected_names:
            self._rename(oldname=filename, error=[NFS4ERR_NOENT,NFS4ERR_INVAL])

    def testDirToObj(self):
        """RENAME should return NFS4ERR_EXIST when renaming dir into existing nondir
        """
        self.init_connection()

        try: self.create_obj()
        except SkipException, e:
            self.skip(e)

        # Create newname as a file
        ops = self.create_via_open(filename=self.newname)
        res = self.ncl.do_ops(ops)
        self.assert_OK(res)

        # Try to rename
        self._rename(error=[NFS4ERR_EXIST])

    def testDirToDir(self):
        """RENAME should allow renaming of dir into existing, empty dir
        """
        self.init_connection()
        try:
            self.create_obj(self.oldname)
            self.create_obj(self.newname)
        except SkipException, e:
            self.skip(e)

        # Try to rename
        self._rename()

    def testObjToDir(self):
        """RENAME should return NFS4ERR_EXIST when renaming nondir into existing dir
        """
        self.init_connection()

        try: self.create_obj(self.newname)
        except SkipException, e:
            self.skip(e)

        # Create oldname as a file
        ops = self.create_via_open(filename=self.oldname)
        res = self.ncl.do_ops(ops)
        self.assert_OK(res)

        # Try to rename
        self._rename(error=[NFS4ERR_EXIST])

    def testFileToFile(self):
        """RENAME should allow renaming of file into existing file
        """
        self.init_connection()
        # Create oldname as a file
        ops = self.create_via_open(filename=self.oldname)
        res = self.ncl.do_ops(ops)
        self.assert_OK(res)

        # Create newname as an object
        ops = self.create_via_open(filename=self.newname)
        res = self.ncl.do_ops(ops)
        self.assert_OK(res)

        # Try to rename
        self._rename()

    def testIntoFullDir(self):
        """RENAME should return NFS4ERR_EXIST or NFS4ERR_NOTEMPTY when renaming into existing, nonempty dir
        """
        self.init_connection()
        try: self.create_obj()
        except SkipException, e:
            self.skip(e)

        self._rename(newdir=self.dirfile, newname="porting",
                     error=[NFS4ERR_EXIST,NFS4ERR_NOTEMPTY])

    def testSelfRename(self):
        """RENAME that does nothing"""
        try: self.create_obj()
        except SkipException, e:
            self.skip(e)
        res = self._rename(newname=self.oldname)
        scinfo = res.resarray[-1].arm.arm.source_cinfo
        tcinfo = res.resarray[-1].arm.arm.target_cinfo
        if scinfo.before != scinfo.after or tcinfo.before != tcinfo.after:
            self.fail("Something was changed")

    def testLinkRename(self):
        """RENAME of file into its hard link should do nothing"""
        self.init_connection()
        try: self.create_obj(self.newname)
        except SkipException, e:
            self.skip(e)
        # Create oldname as a file
        ops = self.create_via_open(filename=self.oldname)
        res = self.ncl.do_ops(ops)
        self.assert_OK(res)
        # Create hardlink to oldname as newname/foo
        ops = [self.ncl.putrootfh_op()]
        ops.extend(self.ncl.lookup_path(self.tmp_dir+[self.oldname]))
        ops.append(self.ncl.savefh_op())
        ops.append(self.ncl.putrootfh_op())
        ops.extend(self.ncl.lookup_path(self.tmp_dir+[self.newname]))
        ops.append(self.ncl.link_op("foo"))
        res = self.ncl.do_ops(ops)
        self.assert_OK(res)

        # Rename
        res = self._rename(newname='foo', newdir=self.tmp_dir+[self.newname])
        scinfo = res.resarray[-1].arm.arm.source_cinfo
        tcinfo = res.resarray[-1].arm.arm.target_cinfo
        if scinfo.before != scinfo.after or tcinfo.before != tcinfo.after:
            self.fail("Something was changed")
