from base_st_classes import *

class SetclientidconfirmSuite(NFSSuite):
    """Test operation 36: SETCLIENTID_CONFIRM

    Equivalence partitioning:

    Input Condition: clientid
        Valid equivalence classes:
            valid clientid(10)
        Invalid equivalence classes:
            stale clientid(11)
    """
    # Override setUp. Just connect, don't do SETCLIENTID etc. 
    def setUp(self):
        self.connect()

    def _set(self, ncl, id, verifier):
        setclientidop = ncl.setclientid(verifier,id)
        res = ncl.do_ops([setclientidop])
        return res

    def _confirm(self, ncl, clientid, verifier):
        setclientid_confirmop = ncl.setclientid_confirm_op(clientid,verifier)
        res = ncl.do_ops([setclientid_confirmop])
        return res
    
    #
    # Testcases covering valid equivalence classes.
    #
    def testValid(self):
        """SETCLIENTID_CONFIRM on valid verifier
        
        Covered valid equivalence classes: 10
        """
        
        # SETCLIENTID
        setclientidop = self.ncl.setclientid()
        res =  self.ncl.do_ops([setclientidop])
        self.assert_OK(res)
        clientid = res.resarray[0].arm.arm.clientid
        idconfirm = res.resarray[0].arm.arm.setclientid_confirm
        
        # SETCLIENTID_CONFIRM
        setclientid_confirmop = self.ncl.setclientid_confirm_op(clientid,idconfirm)
        res =  self.ncl.do_ops([setclientid_confirmop])
        self.assert_OK(res)

    #
    # Testcases covering invalid equivalence classes.
    #
    def testStale(self):
        """SETCLIENTID_CONFIRM on stale vrf should return NFS4ERR_STALE_CLIENTID

        Covered invalid equivalence classes: 11
        """
        clientid = self.get_invalid_clientid()
        setclientid_confirmop = self.ncl.setclientid_confirm_op(clientid,"")
        res =  self.ncl.do_ops([setclientid_confirmop])
        self.assert_status(res, [NFS4ERR_STALE_CLIENTID])

    def testBadConfirm(self):
        """SETCLIENTID_CONFIRM with case not covered in RFC

        Note this returns INVAL (on current CITI server),
        which is not on list of returned errors.
        """

        id = self.ncl.gen_uniq_id() + "setclientidconfirm.testBadConfirm"
        verifier = self.ncl.gen_random_64()

        # 1st SETCLIENTID + SETCLIENTID_CONFIRM
        res = self._set(self.ncl, id, verifier)
        self.assert_OK(res)
        clientid = res.resarray[0].arm.arm.clientid
        idconfirm = res.resarray[0].arm.arm.setclientid_confirm
        res = self._confirm(self.ncl, clientid, idconfirm)
        self.assert_OK(res)

        # 2nd SETCLIENTID 
        res = self._set(self.ncl, id, verifier)
        self.assert_OK(res)

        # Confirm 1st set again, instead of 2nd
        res = self._confirm(self.ncl, clientid, idconfirm)
        self.assert_status(res,[]) #What is approp error?

    def testAllCases(self):
        """SETCLIENTID_CONFIRM with each case from RFC """

        id = self.ncl.gen_uniq_id() + "setclientidconfirm.testAllCases"
        verifier = self.ncl.gen_random_64()

        # no (**c*s), no (**c*s)
        clientid = self.get_invalid_clientid()
        idconfirm = "abcdefgh"
        res = self._confirm(self.ncl, clientid, idconfirm)
        self.assert_status(res, [NFS4ERR_STALE_CLIENTID])

        # no (**c*s) and no (*xd*t), (*xc*s)
        res = self._set(self.ncl, id, verifier)
        self.assert_OK(res)
        clientid = res.resarray[0].arm.arm.clientid
        idconfirm = res.resarray[0].arm.arm.setclientid_confirm
        res = self._confirm(self.ncl, clientid, idconfirm)
        self.assert_OK(res)

        # no (**c*s) and (*xd*t), (*xc*s)
        verifier = self.ncl.gen_random_64()
        res = self._set(self.ncl, id, verifier)
        self.assert_OK(res)
        clientid = res.resarray[0].arm.arm.clientid
        idconfirm = res.resarray[0].arm.arm.setclientid_confirm
        res = self._confirm(self.ncl, clientid, idconfirm)
        self.assert_OK(res)

        # (vxc*s), no (vxc**)
        res = self._confirm(self.ncl, clientid, idconfirm)
        self.assert_OK(res)

        # (vxc*t), (vxc*s)
        res = self._set(self.ncl, id, verifier)
        self.assert_OK(res)
        clientid = res.resarray[0].arm.arm.clientid
        idconfirm = res.resarray[0].arm.arm.setclientid_confirm
        res = self._confirm(self.ncl, clientid, idconfirm)
        self.assert_OK(res)
        

    def testLotsOfClients(self):
        """SETCLIENTID_CONFIRM called multiple times"""
        baseid =  self.ncl.gen_uniq_id()
        for i in range(1024):
            # SETCLIENTID
            id = baseid + str(i)
            verifier = self.ncl.gen_random_64()
            setidop = self.ncl.setclientid(verifier,id)
            res = self.ncl.do_ops([setidop])
            self.assert_OK(res)
            # CONFIRM
            clientid = res.resarray[0].arm.arm.clientid
            idconfirm = res.resarray[0].arm.arm.setclientid_confirm
            confirmop = self.ncl.setclientid_confirm_op(clientid,idconfirm)
            res = self.ncl.do_ops([confirmop])
            self.assert_OK(res)
